@@ -1,6 +1,6 @@
 /* top.h - Header file:         show Linux processes */
 /*
- * Copyright (c) 2002-2012, by: James C. Warner
+ * Copyright (c) 2002-2013, by: James C. Warner
  *    All rights reserved.      8921 Hilloway Road
  *                              Eden Prairie, Minnesota 55347 USA
  *
@@ -24,6 +24,7 @@
 
         /* Defines represented in configure.ac ----------------------------- */
 //#define OOMEM_ENABLE            /* enable the SuSE out-of-memory additions */
+//#define SIGNALS_LESS            /* favor reduced signal load over response */
 
         /* Development/Debugging defines ----------------------------------- */
 //#define ATEOJ_RPTHSH            /* report on hash specifics, at end-of-job */
@@ -44,10 +45,11 @@
 //#define PRETEND4CPUS            /* pretend we're smp with 4 ticsers (sic)  */
 //#define PRETENDNOCAP            /* use a terminal without essential caps   */
 //#define RCFILE_NOERR            /* rcfile errs silently default, vs. fatal */
+//#define RECALL_FIXED            /* don't reorder saved strings if recalled */
 //#define RMAN_IGNORED            /* don't consider auto right margin glitch */
 //#define SCROLLVAR_NO            /* disable intra-column horizontal scroll  */
 //#define STRINGCASENO            /* case insenstive compare/locate versions */
-//#define TERMIO_PROXY            /* true line editing, beyond native input  */
+//#define TERMIOS_ONLY            /* just limp along with native input only  */
 //#define TREE_NORESET            /* sort keys do NOT force forest view OFF  */
 //#define USE_X_COLHDR            /* emphasize header vs. whole col, for 'x' */
 //#define VALIDATE_NLS            /* validate the integrity of all nls tbls  */
@@ -78,9 +80,10 @@
 #define linux_version_code LINUX_VERSION(2,5,43)
 #endif
 
-#ifdef STRINGCASENO
    // pretend as if #define _GNU_SOURCE
 char *strcasestr(const char *haystack, const char *needle);
+
+#ifdef STRINGCASENO
 #define STRSTR  strcasestr
 #define STRCMP  strcasecmp
 #else
@@ -96,8 +99,8 @@ char *strcasestr(const char *haystack, const char *needle);
 
         /* Length of time a message is displayed and the duration
            of a 'priming' wait during library startup (in microseconds) */
-#define MSG_USLEEP  (useconds_t)1250000
-#define LIB_USLEEP  (useconds_t)150000
+#define MSG_USLEEP  1250000
+#define LIB_USLEEP  150000
 
         /* Specific process id monitoring support (command line only) */
 #define MONPIDMAX  20
@@ -126,7 +129,7 @@ char *strcasestr(const char *haystack, const char *needle);
 #define ROWMAXSIZ  ( SCREENMAX + 16 * (CAPBUFSIZ + CLRBUFSIZ) )
    // minimum size guarantee for dynamically acquired 'readfile' buffer
 #define READMINSZ  2048
-   // size of preallocated search string buffers, same as linein()
+   // size of preallocated search string buffers, same as ioline()
 #define FNDBUFSIZ  MEDBUFSIZ
 
 
@@ -137,20 +140,21 @@ char *strcasestr(const char *haystack, const char *needle);
 #define COLPLUSCH   '+'
 
    // support for keyboard stuff (cursor motion keystrokes, mostly)
-#define kbd_ENTER  '\n'
 #define kbd_ESC    '\033'
 #define kbd_SPACE  ' '
-#define kbd_UP     '\x81'
-#define kbd_DOWN   '\x82'
-#define kbd_RIGHT  '\x83'
-#define kbd_LEFT   '\x84'
-#define kbd_PGUP   '\x85'
-#define kbd_PGDN   '\x86'
-#define kbd_END    '\x87'
-#define kbd_HOME   '\x88'
-#define kbd_BKSP   '\x89'
-#define kbd_INS    '\x8a'
-#define kbd_DEL    '\x8b'
+#define kbd_ENTER  '\n'
+#define kbd_UP     129
+#define kbd_DOWN   130
+#define kbd_LEFT   131
+#define kbd_RIGHT  132
+#define kbd_PGUP   133
+#define kbd_PGDN   134
+#define kbd_HOME   135
+#define kbd_END    136
+#define kbd_BKSP   137
+#define kbd_INS    138
+#define kbd_DEL    139
+#define kbd_CtrlO  '\017'
 
         /* Special value in Pseudo_row to force an additional procs refresh
            -- used at startup and for task/thread mode transitions */
@@ -183,6 +187,7 @@ enum pflag {
 #endif
    P_ENV,
    P_FV1, P_FV2,
+   P_USE,
 #ifdef USE_X_COLHDR
    // not really pflags, used with tbl indexing
    P_MAXPFLGS
@@ -197,6 +202,11 @@ enum scale_enum {
    SK_Kb, SK_Mb, SK_Gb, SK_Tb, SK_Pb, SK_Eb, SK_SENTINEL
 };
 
+        /* Used to manipulate (and document) the Frames_resize states */
+enum resize_states {
+   RESIZ_clr, RESIZ_kbd, RESIZ_sig
+};
+
         /* This typedef just ensures consistent 'process flags' handling */
 typedef unsigned char FLG_t;
 
@@ -298,6 +308,10 @@ typedef struct CPU_t {
         // these flag(s) have no command as such - they're for internal use
 #define INFINDS_xxx  0x010000     // build rows for find_string, not display
 #define EQUWINS_xxx  0x000001     // rebalance all wins & tasks (off i,n,u/U)
+#ifndef USE_X_COLHDR
+#define NOHISEL_xxx  0x200000     // restrict Show_HICOLS for osel temporarily
+#define NOHIFND_xxx  0x100000     // restrict Show_HICOLS for find temporarily
+#endif
 
         // Default flags if there's no rcfile to provide user customizations
 #define DEF_WINFLGS ( View_LOADAV | View_STATES | View_CPUSUM | View_MEMORY \
@@ -362,6 +376,7 @@ typedef struct WIN_t {
           varcolsz,        // max length of variable width column(s)
           usrseluid,       // validated uid for 'u/U' user selection
           usrseltyp,       // the basis for matching above uid
+          usrselflg,       // flag denoting include/exclude matches
           hdrcaplen;       // column header xtra caps len, if any
    char   capclr_sum [CLRBUFSIZ],      // terminfo strings built from
           capclr_msg [CLRBUFSIZ],      //   RCW_t colors (& rebuilt too),
@@ -377,6 +392,9 @@ typedef struct WIN_t {
           columnhdr [SCREENMAX],       // column headings for procflgs
 #endif
          *captab [CAPTABMAX];          // captab needed by show_special()
+   struct osel_s *osel_1st;            // other selection criteria anchor
+   int    osel_tot;                    // total of other selection criteria
+   char  *osel_prt;                    // other stuff printable as status line
    char  *findstr;                     // window's current/active search string
    int    findlen;                     // above's strlen, without call overhead
    proc_t **ppt;                       // this window's proc_t ptr array
@@ -451,6 +469,11 @@ typedef struct WIN_t {
       if ( (*P)->n < (*Q)->n ) return SORT_lt; \
       if ( (*P)->n > (*Q)->n ) return SORT_gt; \
       return SORT_eq; }
+#define SCB_NUM2(f,n1,n2) \
+   static int SCB_NAME(f) (const proc_t **P, const proc_t **Q) { \
+      if ( ((*P)->n1+(*P)->n2) < ((*Q)->n1+(*Q)->n2) ) return SORT_lt; \
+      if ( ((*P)->n1+(*P)->n2) > ((*Q)->n1+(*Q)->n2) ) return SORT_gt; \
+      return SORT_eq; }
 #define SCB_NUMx(f,n) \
    static int SCB_NAME(f) (const proc_t **P, const proc_t **Q) { \
       return Frame_srtflg * ( (*Q)->n - (*P)->n ); }
@@ -515,7 +538,7 @@ typedef struct WIN_t {
 
         /* Orderly end, with any sort of message - see fmtmk */
 #define debug_END(s) { \
-           static void error_exit (const char *); \
+           void error_exit (const char *); \
            fputs(Cap_clr_scr, stdout); \
            error_exit(s); \
         }
@@ -573,7 +596,7 @@ typedef struct WIN_t {
    { P_UEN, DEF_WINFLGS, 0, \
       COLOR_YELLOW, COLOR_YELLOW, COLOR_GREEN, COLOR_YELLOW, \
       "Usr", USR_FIELDS } \
-   }, 0 }
+   }, 0, SK_Kb, SK_Kb, 0 }
 
         /* Summary Lines specially formatted string(s) --
            see 'show_special' for syntax details + other cautions. */
@@ -587,9 +610,15 @@ typedef struct WIN_t {
 #if defined(ATEOJ_RPTHSH) && defined(OFF_HST_HASH)
 # error 'ATEOJ_RPTHSH' conflicts with 'OFF_HST_HASH'
 #endif
+#if defined(RECALL_FIXED) && defined(TERMIOS_ONLY)
+# error 'RECALL_FIXED' conflicts with 'TERMIOS_ONLY'
+#endif
 #if (LRGBUFSIZ < SCREENMAX)
 # error 'LRGBUFSIZ' must NOT be less than 'SCREENMAX'
 #endif
+#if defined(TERMIOS_ONLY)
+# warning 'TERMIOS_ONLY' disables input recall and makes man doc incorrect
+#endif
 
 
 /*######  Some Prototypes (ha!)  #########################################*/
@@ -608,7 +637,6 @@ typedef struct WIN_t {
 //atic void          bye_bye (const char *str);
 //atic void          error_exit (const char *str);
 //atic void          library_err (const char *fmts, ...);
-//atic void          pause_pgm (void);
 //atic void          sig_abexit (int sig);
 //atic void          sig_endpgm (int dont_care_sig);
 //atic void          sig_paused (int dont_care_sig);
@@ -623,16 +651,20 @@ typedef struct WIN_t {
 /*------  Low Level Memory/Keyboard/File I/O support  --------------------*/
 //atic void         *alloc_c (size_t num);
 //atic void         *alloc_r (void *ptr, size_t num);
-//atic int           chin (int ech, char *buf, unsigned cnt);
-//atic int           keyin (int init);
-//atic char         *linein (const char *prompt);
+//atic char         *alloc_s (const char *str);
+//atic inline int    ioa (struct timespec *ts);
+//atic int           ioch (int ech, char *buf, unsigned cnt);
+//atic int           iokey (int action);
+//atic char         *ioline (const char *prompt);
 //atic int           readfile (FILE *fp, char **baddr, size_t *bsize, size_t *bread);
 /*------  Small Utility routines  ----------------------------------------*/
 //atic float         get_float (const char *prompt);
 //atic int           get_int (const char *prompt);
 //atic inline const char *hex_make (KLONG num, int noz);
+//atic void          osel_clear (WIN_t *q);
+//atic inline int    osel_matched (const WIN_t *q, FLG_t enu, const char *str);
 //atic const char   *user_certify (WIN_t *q, const char *str, char typ);
-//atic inline int    user_matched (WIN_t *q, const proc_t *p);
+//atic inline int    user_matched (const WIN_t *q, const proc_t *p);
 /*------  Basic Formatting support  --------------------------------------*/
 //atic inline const char *justify_pad (const char *str, int width, int justr);
 //atic inline const char *make_chr (const char ch, int width, int justr);
@@ -683,18 +715,21 @@ typedef struct WIN_t {
 //atic void          whack_terminal (void);
 /*------  Windows/Field Groups support  ----------------------------------*/
 //atic void          win_names (WIN_t *q, const char *name);
-//atic WIN_t        *win_select (char ch);
+//atic void          win_reset (WIN_t *q);
+//atic WIN_t        *win_select (int ch);
 //atic int           win_warn (int what);
 //atic void          wins_clrhlp (WIN_t *q, int save);
 //atic void          wins_colors (void);
 //atic void          wins_reflag (int what, int flg);
 //atic void          wins_stage_1 (void);
 //atic void          wins_stage_2 (void);
-/*------  Interactive Input support (do_key helpers)  --------------------*/
-//atic void          file_writerc (void);
+/*------  Interactive Input Tertiary support  ----------------------------*/
 //atic inline int    find_ofs (const WIN_t *q, const char *buf);
 //atic void          find_string (int ch);
 //atic void          help_view (void);
+//atic void          other_selection (int ch);
+//atic void          write_rcfile (void);
+/*------  Interactive Input Secondary support (do_key helpers)  ----------*/
 //atic void          keys_global (int ch);
 //atic void          keys_summary (int ch);
 //atic void          keys_task (int ch);
