@@ -1,6 +1,6 @@
 /* top_nls.c - provide the basis for future nls translations */
 /*
- * Copyright (c) 2011-2012,     by: James C. Warner
+ * Copyright (c) 2011-2013,     by: James C. Warner
  *    All rights reserved.      8921 Hilloway Road
  *                              Eden Prairie, Minnesota 55347 USA
  *
@@ -19,9 +19,6 @@
 
 #include <locale.h>
 #include <stdio.h>
-#ifdef VALIDATE_NLS
-#include <stdlib.h>
-#endif
 #include <string.h>
 
 #include "../include/nls.h"
@@ -29,6 +26,10 @@
 #include "top.h"
 #include "top_nls.h"
 
+#ifdef VALIDATE_NLS
+#include <stdlib.h>
+#endif
+
         // Programmer Note(s):
         //  Preparation ---------------------------------------------
         //    Unless you have *something* following the gettext macro,
@@ -66,9 +67,10 @@
         //                           with "... charset=UTF-8\n"
         //        . translate msgstr values, leaving msgid unchanged
         //    msgfmt ll_CC.po --strict --output-file=procps-ng.mo
-        //     . after which chmod 644
-        //     . move to /usr/share/local/ll_CC/LC_MESSAGES/ directory
-        //
+        //     . after which ensure, chmod 644
+        //     . then copy
+        //         to /usr/share/locale-langpack/ll_CC/LC_MESSAGES/
+        //         or /usr/share/locale/ll_CC/LC_MESSAGES/
         //  Testing -------------------------------------------------
         //    export LC_ALL= && export LANGUAGE=ll_CC
         //    run some capable program like top
@@ -254,6 +256,9 @@ static void build_two_nlstabs (void) {
 /* Translation Hint: maximum 'vMn' = 3 */
    Head_nlstab[P_FV2] = _("vMn");
    Desc_nlstab[P_FV2] = _("Minor Faults delta");
+/* Translation Hint: maximum 'USED' = 4 */
+   Head_nlstab[P_USE] = _("USED");
+   Desc_nlstab[P_USE] = _("Res+Swap Size (KiB)");
 }
 
 
@@ -298,13 +303,12 @@ static void build_norm_nlstab (void) {
    Norm_nlstab[ON_word_only_txt] = _("On");
    Norm_nlstab[OFF_one_word_txt] = _("Off");
 /* Translation Hint: Only the following words should be translated
-   .                 delay, limit, user, cols (abbreviation for columns)*/
+   .                 secs (seconds), max (maximum), user, field, cols (columns)*/
    Norm_nlstab[USAGE_abbrev_txt] = _(" -hv | -bcHiOSs -d secs -n max -u|U user -p pid(s) -o field -w [cols]");
    Norm_nlstab[FAIL_statget_txt] = _("failed /proc/stat read");
    Norm_nlstab[FOREST_modes_fmt] = _("Forest mode %s");
    Norm_nlstab[FAIL_tty_get_txt] = _("failed tty get");
-   Norm_nlstab[FAIL_tty_mod_fmt] = _("failed Tty_tweaked set: %s");
-   Norm_nlstab[FAIL_tty_raw_fmt] = _("failed Tty_raw set: %s");
+   Norm_nlstab[FAIL_tty_set_fmt] = _("failed tty set: %s");
    Norm_nlstab[CHOOSE_group_txt] = _("Choose field group (1 - 4)");
    Norm_nlstab[DISABLED_cmd_txt] = _("Command disabled, 'A' mode required");
    Norm_nlstab[DISABLED_win_fmt] = _("Command disabled, activate %s with '-' or '_'");
@@ -333,8 +337,6 @@ static void build_norm_nlstab (void) {
    Norm_nlstab[FAIL_alloc_r_txt] = _("failed memory re-allocate");
    Norm_nlstab[BAD_numfloat_txt] = _("Unacceptable floating point");
    Norm_nlstab[BAD_username_txt] = _("Invalid user");
-   Norm_nlstab[FAIL_sigstop_fmt] = _("failed sigprocmask, SIG_BLOCK: %s");
-   Norm_nlstab[FAIL_sigmask_fmt] = _("failed sigprocmask, SIG_SETMASK: %s");
    Norm_nlstab[FOREST_views_txt] = _("forest view");
    Norm_nlstab[FAIL_widepid_txt] = _("failed pid maximum size test");
    Norm_nlstab[FAIL_widecpu_txt] = _("failed number of cpus test");
@@ -378,7 +380,8 @@ static void build_norm_nlstab (void) {
    Norm_nlstab[FIND_no_find_fmt] = _("%s\"%s\" not found");
    Norm_nlstab[XTRA_fixwide_fmt] = _("width incr is %d, change to (0 default, -1 auto)");
    Norm_nlstab[XTRA_warncfg_txt] = _("Overwrite existing obsolete/corrupted rcfile?");
-   Norm_nlstab[XTRA_sortopt_fmt] = _("unrecognized field name '%s'");
+   Norm_nlstab[XTRA_badflds_fmt] = _("unrecognized field name '%s'");
+   Norm_nlstab[XTRA_winsize_txt] = _("even using field names only, window is now too small");
 #ifndef INSP_OFFDEMO
    Norm_nlstab[YINSP_demo01_txt] = _("Open Files");
    Norm_nlstab[YINSP_demo02_txt] = _("NUMA Info");
@@ -427,6 +430,18 @@ static void build_norm_nlstab (void) {
    Norm_nlstab[YINSP_rcfile_fmt] = _("could not parse rcfile inspect entry %d");
    Norm_nlstab[YINSP_status_fmt] = _("%s: %*d-%-*d lines, %*d-%*d columns, %lu bytes read");
    Norm_nlstab[YINSP_workin_txt] = _("patience please, working...");
+/* Translation Hint: Below are 2 abbreviations which can be as long as needed:
+   .                 FLD = FIELD, VAL = VALUE */
+   Norm_nlstab[OSEL_prompts_fmt] = _("add filter #%d (%s) as: [!]FLD?VAL");
+   Norm_nlstab[OSEL_casenot_txt] = _("ignoring case");
+   Norm_nlstab[OSEL_caseyes_txt] = _("case sensitive");
+   Norm_nlstab[OSEL_errdups_txt] = _("duplicate filter was ignored");
+   Norm_nlstab[OSEL_errdelm_fmt] = _("'%s' filter delimiter is missing");
+   Norm_nlstab[OSEL_errvalu_fmt] = _("'%s' filter value is missing");
+   Norm_nlstab[WORD_include_txt] = _("include");
+   Norm_nlstab[WORD_exclude_txt] = _("exclude");
+   Norm_nlstab[OSEL_statlin_fmt] = _("<Enter> to resume, filters: %s");
+   Norm_nlstab[WORD_noneone_txt] = _("none");
 }
 
 
@@ -474,8 +489,8 @@ static void build_uniq_nlstab (void) {
       "  c,i,S,j . Toggle: '~1c~2' Cmd name/line; '~1i~2' Idle; '~1S~2' Time; '~1j~2' Str justify\n"
       "  x~5,~1y~5     . Toggle highlights: '~1x~2' sort field; '~1y~2' running tasks\n"
       "  z~5,~1b~5     . Toggle: '~1z~2' color/mono; '~1b~2' bold/reverse (only if 'x' or 'y')\n"
-      "  u,U     . Filter by: '~1u~2' effective user; '~1U~2' effective/file/real/saved user\n"
-      "  n or #  . Set maximum tasks displayed\n"
+      "  u,U,o,O . Filter by: '~1u~2'/'~1U~2' effective/any user; '~1o~2'/'~1O~2' other criteria\n"
+      "  n,#,^O  . Set: '~1n~2'/'~1#~2' max tasks displayed; Show: ~1Ctrl~2+'~1O~2' other filter(s)\n"
       "  C,...   . Toggle scroll coordinates msg for: ~1up~2,~1down~2,~1left~2,~1right~2,~1home~2,~1end~2\n"
       "\n"
       "%s"
@@ -483,7 +498,7 @@ static void build_uniq_nlstab (void) {
       "  q         Quit\n"
       "          ( commands shown with '.' require a ~1visible~2 task display ~1window~2 ) \n"
       "Press '~1h~2' or '~1?~2' for help with ~1Windows~2,\n"
-      "any other key to continue ");
+      "Type 'q' or <Esc> to continue ");
 
 /* Translation Hint: As is true for the text above, the "keys" shown to the left and
    .                 also imbedded in the translatable text (along with escape seqs)
@@ -557,21 +572,21 @@ static void build_uniq_nlstab (void) {
       "   available toggles: ~1B~2 =disable bold globally (~1%s~2),\n"
       "       ~1z~2 =color/mono (~1%s~2), ~1b~2 =tasks \"bold\"/reverse (~1%s~2)\n"
       "\n"
-      "Select ~1target~2 as upper case letter:\n"
+      "1) Select a ~1target~2 as an upper case letter, ~1current target~2 is ~1 %c ~4:\n"
       "   S~2 = Summary Data,~1  M~2 = Messages/Prompts,\n"
       "   H~2 = Column Heads,~1  T~2 = Task Information\n"
-      "Select ~1color~2 as number:\n"
+      "2) Select a ~1color~2 as a number, ~1current color~2 is ~1 %d ~4:\n"
       "   0~2 = black,~1  1~2 = red,    ~1  2~2 = green,~1  3~2 = yellow,\n"
       "   4~2 = blue, ~1  5~2 = magenta,~1  6~2 = cyan, ~1  7~2 = white\n"
       "\n"
-      "Selected: ~1target~2 ~1 %c ~4; ~1color~2 ~1 %d ~4\n"
-      "   press 'q' to abort changes to window '~1%s~2'\n"
-      "   press 'a' or 'w' to commit & change another, <Enter> to commit and end ");
+      "3) Then use these keys when finished:\n"
+      "   'q' to abort changes to window '~1%s~2'\n"
+      "   'a' or 'w' to commit & change another, <Enter> to commit and end ");
 
    Uniq_nlstab[FIELD_header_fmt] = _(""
       "Fields Management~2 for window ~1%s~6, whose current sort field is ~1%s~2\n"
       "   Navigate with Up/Dn, Right selects for move then <Enter> or Left commits,\n"
-      "   'd' or <Space> toggles display, 's' sets sort.  Use 'q' or <Esc> to end! ");
+      "   'd' or <Space> toggles display, 's' sets sort.  Use 'q' or <Esc> to end!\n");
 
    Uniq_nlstab[STATE_line_1_fmt] = _("%s:~3"
       " %3u ~2total,~3 %3u ~2running,~3 %3u ~2sleeping,~3 %3u ~2stopped,~3 %3u ~2zombie~3\n");
@@ -668,16 +683,6 @@ void initialize_nls (void) {
          fprintf(stderr, nls_err, "Uniq", i);
          exit(1);
       }
- #ifndef INSP_OFFDEMO
-   if (READMINSZ < strlen(N_txt(YINSP_dstory_txt)) +1) {
-      fprintf(stderr
-         , "\nAssertion Failed in %s (%s):\n"
-            "\t'READMINSZ < strlen(N_txt(YINSP_dstory_txt)) + 1'\n"
-            "READMINSZ must be at least %u !\n\n"
-         , __FILE__,  __func__, (unsigned)strlen(N_txt(YINSP_dstory_txt)) + 1);
-         exit(1);
-   }
- #endif
 #else
    setlocale (LC_ALL, "");
    bindtextdomain(PACKAGE, LOCALEDIR);
